#!/usr/bin/env python

'''manipulate CIGAR string represented as list of tuples (BAM file)
BAM	OP	Description
0	M	alignment match
1	I	insertion to read. (relative to reference genome)
2	D	deletion from read. (relative to reference genome)
3	N	skipped region from the reference
4	S	soft clipping (clipped sequence present in SEQ)
5	H	hard clipping (clipped sequences NOT present in SEQ) 
6	P	padding (silent deletion from padded reference)
7	=	sequence match
8	X	sequence mismatch

Example:
The tuple [ (0,3), (1,5), (0,2) ] refers to an alignment with 3 matches, 
5 insertions and another 2 matches.
 
NOTE: only deal with Match, Gap, Soft Clip, Insertion, Deletion 
'''

__author__ = "Liguo Wang"
__copyright__ = "Copyright 2012 Mayo Clinic"
__credits__ = []
__license__ = "GPL"
__version__ = "1.0.1"
__maintainer__ = "Liguo Wang"
__email__ = "wangliguo78@gmail.com"
__status__ = "Development" #Prototype or Production


def map_bounds(start_pos, cigar):
	'''return the start, end corrdinates (on genome) of mapped region'''
	span_size = 0	#the actual genomic region size that covered by read
	for c,s in cigar:	#code and size
		if c == 0:	#match
			span_size += s
		elif c == 1:	#insertion to the ref
			continue
		elif c == 2: # deleltion
			span_size += s
		elif c == 3:	# skipping
			span_size += s
		else:
			continue
	return (start_pos, start_pos+span_size)
		

def fetch_exon(chrom, st, cigar):
	''' fetch exon regions defined by cigar. st must be zero based
	return list of tuple of (chrom,st, end)
	'''
	#match = re.compile(r'(\d+)(\D)')
	chrom_st = st
	exon_bound =[]
	for c,s in cigar:	#code and size
		if c==0:		#match
			exon_bound.append((chrom, chrom_st,chrom_st + s))
			chrom_st += s
		elif c==1:		#insertion to ref
			continue
		elif c==2:		#deletion to ref
			chrom_st += s
		elif c==3:		#gap or intron
			chrom_st += s
		elif c==4:		#soft clipping. We do NOT include soft clip as part of exon
			chrom_st += s
		else:
			continue
	return exon_bound
	
def fetch_intron(chrom, st, cigar):
	''' fetch intron regions defined by cigar. st must be zero based
	return list of tuple of (chrom,st, end)
	'''
	#match = re.compile(r'(\d+)(\D)')
	chrom_st = st
	intron_bound =[]
	for c,s in cigar:	#code and size
		if c==0:		#match
			chrom_st += s
		elif c==1:		#insertion to ref
			continue
		elif c==2:		#deletion to ref
			chrom_st += s
		elif c==3:		#gap or intron
			intron_bound.append((chrom, chrom_st,chrom_st+s))
			chrom_st += s
		elif c==4:		#soft clipping. We do NOT include soft clip as part of exon
			#chrom_st += s
			continue
		else:
			continue
	return intron_bound	

def fetch_clip(chrom, st, cigar):
	''' fetch head soft clip regions defined by cigar. st must be zero based
	return list of tuple of (chrom,st, end)
	'''
	#match = re.compile(r'(\d+)(\D)')
	chrom_st = st
	clip_bound =[]
	for c,s in cigar:	#code and size
		if c==0:		#match
			chrom_st += s
		elif c==1:		#insertion to ref
			continue
		elif c==2:		#deletion to ref
			chrom_st += s
		elif c==3:		#gap or intron
			chrom_st += s
		elif c==4:		#soft clipping. We do NOT include soft clip as part of exon
			clip_bound.append((chrom, chrom_st, chrom_st + s))
			chrom_st += s
		else:
			continue
	return clip_bound		

def fetch_deletion(chrom, st, cigar):
	''' fetch deletion regions defined by cigar. st must be zero based
	return list of tuple of (chrom,st, end)
	'''
	#match = re.compile(r'(\d+)(\D)')
	chrom_st = st
	del_bound =[]
	for c,s in cigar:	#code and size
		if c==0:		#match
			chrom_st += s
		elif c==1:		#insertion to ref
			continue
		elif c==2:		#deletion to ref
			del_bound.append((chrom, chrom_st, chrom_st + s))
			chrom_st += s
		elif c==3:		#gap or intron
			chrom_st += s
		elif c==4:		#soft clipping. We do NOT include soft clip as part of exon
			chrom_st += s
		else:
			continue
	return del_bound			

def fetch_deletion_range(cigar):
	''' fetch deletion regions defined by cigar. st must be zero based
	return list of tuple of (st, end). 'st','end' is relative to the 
	start of read.
	'''
	del_bound =[]
	st = 0
	for c,s in cigar:	#code and size
		if c==0:		#match
			st += s
		elif c==4:
			st += s		#soft clip
		elif c==1:		#insertion to ref
			st += s
		elif c==2:		#deletion to ref
			del_bound.append((st,s))	#only record the start position of deletion, and the deletion size
		elif c==3:		#gap or intron
			continue
		else:
			continue
	return del_bound


def fetch_insertion_range(cigar):
	''' fetch insertion regions defined by cigar. st must be zero based
	return list of tuple of (st, end). 'st','end' is relative to the 
	start of read.
	'''
	ins_bound =[]
	st = 0
	for c,s in cigar:	#code and size
		if c==0:		#match
			st += s
		elif c==4:
			st += s		#soft clip
		elif c==1:		#insertion to ref
			ins_bound.append((st, s))
			st += s
		elif c==2:		#deletion to ref
			continue
		elif c==3:		#gap or intron
			continue
		else:
			continue
	return ins_bound


	
def fetch_insertion(chrom, st, cigar):
	''' fetch insertion regions defined by cigar. st must be zero based
	return list of tuple of (chrom,st, end)
	
	NOTE: insertion region does NOT present in reference genome and there
	fore cannot represented using reference coordinates[start, end]. So we
	use [start, SIZE).
	[(100,2)] means 2nt insert between 100 and 101
	'''
	#match = re.compile(r'(\d+)(\D)')
	chrom_st = st
	ins_bound =[]
	for c,s in cigar:	#code and size
		if c==0:		#match
			chrom_st += s
		elif c==1:		#insertion to ref
			ins_bound.append((chrom, chrom_st, s))
			continue
		elif c==2:		#deletion to ref
			chrom_st += s
		elif c==3:		#gap or intron
			chrom_st += s
		elif c==4:		#soft clipping. We do NOT include soft clip as part of exon
			chrom_st += s
		else:
			continue
	return ins_bound			


def list2str (lst):
	'''translate samtools returned cigar_list into cigar_string
	
	[(4, 1), (0, 9)] ==> '1S9M'
	'''
	code2Char={'0':'M','1':'I','2':'D','3':'N','4':'S','5':'H','6':'P','7':'=','8':'X'}
        
	cigar_str=''
	for i in lst:
		cigar_str += str(i[1]) + code2Char[str(i[0])]
	return cigar_str

def list2longstr (lst):
	'''translate samtools returned cigar_list into LONG cigar_string
	Sum of lengths of the M/I/S/=/X operations shall equal the length of SEQ
	'''
	
	code2Char={'0':'M','1':'I','2':'D','3':'N','4':'S','5':'H','6':'P','7':'=','8':'X'}
        
	cigar_str=''
	for i in lst:
		(code,length) = (i[0],i[1])
		if code in (0,1,4,7,8):
			cigar_str += code2Char[str(code)] * int(length)
		else:
			continue
	return cigar_str

	


#!/usr/bin/env python
'''calculate coding potential'''

# Fickett TESTCODE data
# NAR 10(17) 5303-531
position_prob ={
'A':[0.94,0.68,0.84,0.93,0.58,0.68,0.45,0.34,0.20,0.22],
'C':[0.80,0.70,0.70,0.81,0.66,0.48,0.51,0.33,0.30,0.23],
'G':[0.90,0.88,0.74,0.64,0.53,0.48,0.27,0.16,0.08,0.08],
'T':[0.97,0.97,0.91,0.68,0.69,0.44,0.54,0.20,0.09,0.09]
}
position_weight={'A':0.26,'C':0.18,'G':0.31,'T':0.33}
position_para  =[1.9,1.8,1.7,1.6,1.5,1.4,1.3,1.2,1.1,0.0]

content_prob={
'A':[0.28,0.49,0.44,0.55,0.62,0.49,0.67,0.65,0.81,0.21],
'C':[0.82,0.64,0.51,0.64,0.59,0.59,0.43,0.44,0.39,0.31],
'G':[0.40,0.54,0.47,0.64,0.64,0.73,0.41,0.41,0.33,0.29],
'T':[0.28,0.24,0.39,0.40,0.55,0.75,0.56,0.69,0.51,0.58]
}
content_weight={'A':0.11,'C':0.12,'G':0.15,'T':0.14}
content_para  =[0.33,0.31,0.29,0.27,0.25,0.23,0.21,0.17,0]

def look_up_position_prob(value, base):
	'''look up positional probability by base and value'''
	if float(value)<0:
		return None
	for idx,val in enumerate (position_para):
		if (float(value) >= val):
			return float(position_prob[base][idx]) * float(position_weight[base])

def look_up_content_prob(value, base):
	'''look up content probability by base and value'''
	if float(value)<0:
		return None
	for idx,val in enumerate (content_para):
		if (float(value) >= val):
			return float(content_prob[base][idx]) * float(content_weight[base])

def fickett_value(dna):
	'''calculate Fickett value. Input is DNA sequence'''
	if (len(dna)<1):
		return None
	fickett_score=0
	dna=dna.upper()
	total_base = len(dna)
	A_content = float(dna.count('A'))/total_base
	C_content = float(dna.count('C'))/total_base
	G_content = float(dna.count('G'))/total_base
	T_content = float(dna.count('T'))/total_base
	#print "A content\t" + str(A_content)
	#print "C content\t" + str(C_content)
	#print "G content\t" + str(G_content)
	#print "T content\t" + str(T_content)
	
	phase_0 = [dna[i] for i in range(0,len(dna)) if i % 3==0]
	phase_1 = [dna[i] for i in range(0,len(dna)) if i % 3==1]
	phase_2 = [dna[i] for i in range(0,len(dna)) if i % 3==2]
	
	A_position=max(phase_0.count('A'),phase_1.count('A'),phase_2.count('A'))/(min(phase_0.count('A'),phase_1.count('A'),phase_2.count('A')) +1.0)
	C_position=max(phase_0.count('C'),phase_1.count('C'),phase_2.count('C'))/(min(phase_0.count('C'),phase_1.count('C'),phase_2.count('C')) +1.0)
	G_position=max(phase_0.count('G'),phase_1.count('G'),phase_2.count('G'))/(min(phase_0.count('G'),phase_1.count('G'),phase_2.count('G')) +1.0)
	T_position=max(phase_0.count('T'),phase_1.count('T'),phase_2.count('T'))/(min(phase_0.count('T'),phase_1.count('T'),phase_2.count('T')) +1.0)
	#print "A position\t" + str(A_position)
	#print "C position\t" + str(C_position)
	#print "G position\t" + str(G_position)
	#print "T position\t" + str(T_position)

	
	#for i (A_content,C_content,G_content,T_content):
	fickett_score += look_up_content_prob(A_content,'A')
	fickett_score += look_up_content_prob(C_content,'C')
	fickett_score += look_up_content_prob(G_content,'G')
	fickett_score += look_up_content_prob(T_content,'T')
	
	fickett_score += look_up_position_prob(A_position,'A')
	fickett_score += look_up_position_prob(C_position,'C')
	fickett_score += look_up_position_prob(G_position,'G')
	fickett_score += look_up_position_prob(T_position,'T')	
	
	return fickett_score
	
	
	


#!/usr/bin/env python
'''manipulate fasta for fastq format files.'''

#import built-in modules
import numpy
import re
import sys
from string import maketrans
from optparse import OptionParser
import collections
from qcmodule import FrameKmer
#import third-party modules

#changes to the paths

#changing history to this module


__author__ = "Liguo Wang"
__copyright__ = "Copyright 2010, Wei Li's Lab"
__credits__ = []
__license__ = "GPL"
__version__ = "1.0.1"
__maintainer__ = "Liguo Wang"
__email__ = "liguow@bcm.edu"
__status__ = "Development" #Prototype or Production



class Fasta:
	'''manipulate fasta or fastq format file
	'''
	
	def __init__(self,fastafile=None):
		'''initialize object, lowercase in sequence is automatically converted into uppercase'''
		self.seqs={}
		self.IDs=[]
		self.transtab = maketrans("ACGTNX","TGCANX")
		self.filename = fastafile
		tmpseq=''
		if fastafile is not None:
			for line in open(fastafile,'r'):
				line=line.strip(' \n')
				if line.startswith('>'):
					if(tmpseq):
						self.seqs[name]=tmpseq
					name=line[1:]
					tmpseq =''
					self.IDs.append(name)
					print >>sys.stderr,"\tloading "+name+' ...'
				else:
					tmpseq += line.upper()
			self.seqs[name]=tmpseq
				
	def addSeq(self,id,seq):
		'''add sequence to current data'''
		if self.seqs.has_key(id):
			print >>sys.stderr,id +" already exists!"
			return
		else:
			self.seqs[id]=seq.upper()
			self.IDs.append(id)
			
	def getNames (self,file=None):
		'''return all sequence IDs'''
		return self.IDs
		
	def getSeq(self,seqID=None):
		'''return sequence for sepcified seqID, otherwise all sequences are returned'''
		if seqID is None:
			return self.seqs.values()
		else:
			return self.seqs[seqID]

	def printSeqs(self,n=50):
		'''print all seqs '''
		for k,v in self.seqs.items():
			print '>' + k
			for i in xrange(0, len(v), n):
				print v[i:i+n]

			
	def getSeqLen(self,seqID=None):
		seqlen=collections.defaultdict(dict)
		if seqID is None:
			for (k,v) in self.seqs.items():
				seqlen[k]=len(v)
		else:
			try:
				seqlen[seqID]=len(self.seqs[seqID])
			except:
				print >>sys.stderr,"Not found"
		return seqlen
			
	def countBase(self,pattern=None):
		'''count occurence of substring (defined by pattern), otherwise count A,C,G,T,N,X
		NOTE: pattern is counted non-overlappingly'''
		if pattern is None:
			print "ID\tTotal\tA\tC\tG\tT\tN\tX"
			for (k,v) in self.seqs.items():
				print k+"\t",
				print len(v),"\t",
				print str(v.count('A'))+"\t",
				print str(v.count('C'))+"\t",
				print str(v.count('G'))+"\t",
				print str(v.count('T'))+"\t",
				print str(v.count('N'))+"\t",
				print v.count('X')
		else:
			for (k,v) in self.seqs.items():
				print k+"\t",
				print str(len(v))+"\t",
				print v.count(pattern)
	def cal_entropy(self,l=3):
		'''calculate entropy for each sequence'''
		for (id,seq) in self.seqs.items():
			entropy = 0
			dna_chars_uniq = FrameKmer.all_possible_kmer(l)
			dna_len = len(seq)
			
			for c in dna_chars_uniq:
				if 'N' in c:
					continue
				prop = seq.count(c)/(1.0*dna_len)
				if prop ==0:
					continue
				information = numpy.log2(1.0/prop)
				entropy += prop * information
			yield(id, entropy)
		
	def revComp(self,seqID=None):
		'''return reverse-complemented sequence for sepcified seqID, otherwise all sequences are 
		reverse-complemented'''
		if seqID is None:
			for (k,v) in self.seqs.items():
				print ">" + k + "_rev"
				tmp = v.translate(self.transtab)
				return tmp[::-1]			
		else:
			return self.seqs[seqID].translate(self.transtab)[::-1]

			
	def getUniqSeqs(self):
		'''remove redundancy from original fasta files.
		duplicated sequences will be only report once'''

		seq2Name={}
		seq2Count={}
		for (key,value) in self.seqs.items():
			seq2Name[value]=key
			if seq2Count.has_key(value):
				seq2Count[value]+=1
			else:
				seq2Count[value]=1
		for value in seq2Name.keys():
				print '>'+ str(seq2Name[value]) + '_' + str(seq2Count[value])
				print value


	def findPattern(self,pat,outfile,seqID=None,rev=True):
		''' find pattern in all sequence unless seqID is specified, coordinates will be returned as bed format file'''
		
		fout=open(outfile,'w')
		length=len(pat)	

		Pat=pat.upper()
		start=0
		
		
		if seqID is None:
			for (k,v) in self.seqs.items():
				loopSwitch=0
				start=0
				while loopSwitch !=-1:
					loopSwitch = v.find(Pat,start)
					if loopSwitch !=-1:
						print >>fout,k + "\t" + str(loopSwitch) + "\t" + str(loopSwitch + length) + "\t" + Pat + "\t0\t+" 
						start = loopSwitch +1
					
		else:
			loopSwitch=0
			start=0
			while loopSwitch !=-1:
				loopSwitch = self.seqs[seqID].find(Pat,start)
				print >>fout, seqID + "\t" + str(loopSwitch) + "\t" + str(loopSwitch + length) + "\t" + Pat + "\t0\t+" 
				start = loopSwitch +1

		if rev==True:
			Pat_rev=Pat.translate(self.transtab)[::-1]
			if seqID is None:
				for (k,v) in self.seqs.items():
					loopSwitch=0
					start=0
					while loopSwitch !=-1:
						loopSwitch = v.find(Pat_rev,start)
						if loopSwitch !=-1:
							print >>fout,k + "\t" + str(loopSwitch) + "\t" + str(loopSwitch + length) + "\t" + Pat + "\t0\t-" 
							start = loopSwitch +1
						
			else:
				loopSwitch=0
				start=0
				while loopSwitch !=-1:
					loopSwitch = self.seqs[seqID].find(Pat_rev,start)
					print >>fout, seqID + "\t" + str(loopSwitch) + "\t" + str(loopSwitch + length) + "\t" + Pat + "\t0\t-" 
					start = loopSwitch +1		

	def fetchSeq(self,chr=None,st=None,end=None,infile=None,outfile=None):
		''' Fetching sequence based on chrName (should be exactly the same as fasta file), St, End. 
		NOTE: the coordinate is 0-based,half-open. use infile to specify multiple coordinates. infile
		should be bed3, bed6 or bed12'''

		if (infile is not None) and (outfile is not None):
			fout=open(outfile,'w')
			for line in open(infile):
				fields=line.strip().split()
				if (len(fields)==3):
					print >>fout,fields[0]+":"+fields[1]+"-"+fields[2]+"\t"+"strand=+"
					print >>fout,self.seqs[fields[0]][int(fields[1]):int(fields[2])].upper()
				elif (len(fields)>3):
					if fields[5]=='-':
						print >>fout,fields[0]+":"+fields[1]+"-"+fields[2]+"\t"+"strand=-"
						print >>fout,self.seqs[fields[0]][int(fields[1]):int(fields[2])].translate(self.transtab)[::-1].upper()
					else:
						print >>fout,fields[0]+":"+fields[1]+"-"+fields[2]+"\t"+"strand=+"
						print >>fout,self.seqs[fields[0]][int(fields[1]):int(fields[2])].upper()
		else:
			try:
				return self.seqs[chr][st:end].upper()
			except:
				print >>sys.stderr, "cannot fetch sequence from " + self.filename + " for " + chr + ":" + str(st) + "-" + str(end)
				return ''
				#print >>sys.stderr, chr + "\t" + str(st) +'\t' + str(end) + "  Please input chr,st,end"
						
def main():
	parser = OptionParser()
	parser.add_option("-i","--input_file",dest="in_file",help="input file name")
	(options,args)=parser.parse_args()
	obj=Fasta(options.in_file)
	obj.printSeqs(n=80)

if __name__ == "__main__":
    main()
	



'''manipulate CIGAR string'''

#import built-in modules
import os,sys
import re
import string
import warnings
import string
import collections
import math
import sets
import random

head_clip = re.compile(r'^(\d+)S')
tail_clip = re.compile(r'(\d+)S$')
insertion = re.compile(r'(\d+)I')
deletion  = re.compile(r'(\d+)D')
matching  = re.compile(r'(\d+)M')
skipping  = re.compile(r'(\d+)N')
read_part = re.compile(r'(\d+)[MIS=X]')
ref_part = re.compile(r'(\d+)[MISND=X]')

prior_insertion = re.compile(r'(.+?)(\d+)I')
prior_deletion = re.compile(r'(.+?)(\d+)D')
prior_intron = re.compile(r'(.+?)(\d+)N')
prior_exon = re.compile(r'(.*?)(\d+)M')

def fetch_head_clip(chr, st, cigar):
	'''return genome coordinate of the head clip part encoded in cigar string
	NOTE: returned coordinates are 0-based.NOTE: st is 0-based'''
	
	block=[]
	chrom_end = int(st)		
	tmp = head_clip.findall(cigar)
	if len(tmp)==0:
		return block
	else:
		chrom_st = chrom_end - int(tmp[0])
		block.append([chr,chrom_st,chrom_end])
	return block

def fetch_tail_clip(chr, st, cigar):
	'''return genome coordinates of the tail clip part encoded in cigar string
	NOTE: returned coordinates are 0-based .  NOTE: st is 0-based'''
	
	block=[]
	h = head_clip.findall(cigar)
	t = tail_clip.findall(cigar)
	if len(t)==0:return block
	else:t_len = int(t[0])
	
	if len(h)==0:h_len=0
	else:h_len=int(h[0])
	ref_length = sum([int(i) for i in ref_part.findall(cigar)])
	#print read_length
	chrom_end = int(st) + (ref_length - h_len)		#because SAM is 1-based
	chrom_st = chrom_end - t_len
	block.append([chr,chrom_st,chrom_end])
	return block
	
def fetch_insertion(chr, st, cigar):
	'''return genome coordinates of the insertion (to reference) encoded in cigar string
	NOTE: returned coordinates are 0-based. Insertion to the reference.  NOTE: st is 0-based'''
	
	block=[]
	h = head_clip.findall(cigar)
	if len(h)==0:h_len=0
	else:h_len=int(h[0])
	
	ref_length=0
	m = prior_insertion.findall(cigar)
	if len(m)==0:
		return block
	else:
		for j in m:
			ref_length += sum([int(i) for i in ref_part.findall(j[0])])
			chrom_st = int(st) + (ref_length - h_len)
			chrom_end = chrom_st + int(j[1])
			block.append([chr,chrom_st,chrom_end])
	return block

def fetch_deletion(chr, st, cigar):
	'''return genome coordinates of the insertion (to reference) encoded in cigar string
	NOTE: returned coordinates are 0-based. Deletion to the reference.  NOTE: st is 0-based'''
	
	block=[]
	h = head_clip.findall(cigar)
	if len(h)==0:h_len=0
	else:h_len=int(h[0])
	
	ref_length=0
	m = prior_deletion.findall(cigar)
	if len(m)==0:
		return block
	else:
		for j in m:
			ref_length += sum([int(i) for i in ref_part.findall(j[0])])
			chrom_st = int(st) + (ref_length - h_len)
			chrom_end = chrom_st + int(j[1])
			block.append([chr,chrom_st,chrom_end])
			ref_length += int(j[1])
	return block

	
def fetch_intron(chr, st, cigar):
	'''return genome coordinates of the introns encoded in cigar string
	NOTE: returned coordinates are 0-based. Deletion to the reference NOTE: 
	st is 0-based '''
	
	block=[]
	h = head_clip.findall(cigar)
	if len(h)==0:h_len=0
	else:h_len=int(h[0])
	
	ref_length=0
	m = prior_intron.findall(cigar)
	if len(m)==0:
		return block
	else:
		for j in m:
			ref_length += sum([int(i) for i in ref_part.findall(j[0])])
			chrom_st = int(st)  + (ref_length - h_len)
			chrom_end = chrom_st + int(j[1])
			block.append([chr,chrom_st,chrom_end])
			ref_length += int(j[1])
	return block
	
def fetch_exon(chr, st, cigar):
	'''return genome coordinates of the exon encoded in cigar string
	NOTE: returned coordinates are 0-based. NOTE: st is 0-based'''
	
	block=[]
	h = head_clip.findall(cigar)
	if len(h)==0:h_len=0
	else:h_len=int(h[0])
	
	ref_length=0
	m = prior_exon.findall(cigar)
	if len(m)==0:
		return block
	else:
		for j in m:
			ref_length += sum([int(i) for i in ref_part.findall(j[0])])
			chrom_st = int(st) + (ref_length - h_len)
			chrom_end = chrom_st + int(j[1])
			block.append([chr,chrom_st,chrom_end])
			ref_length += int(j[1])
	return block
	
def list2str (lst):
	'''translate samtools returned cigar_list into cigar_string'''
	code2Char={'0':'M','1':'I','2':'D','3':'N','4':'S','5':'H','6':'P','7':'=','8':'X'}
	
	cigar_str=''
	for i in lst:
		cigar_str += str(i[1]) + code2Char[str(i[0])]
	return cigar_str



#!/usr/bin/env python
'''-------------------------------------------------------------------------------------------------
For each gene, check whether the RPKM value was saturated or not. Saturation analysis is based on 
re-sampling. For example, sample 5%, 10%, ... , 95%, 100% from total mapped reads, then 
calculate RPKM value for each step. Strand specific sequencing protocol is supported.
-------------------------------------------------------------------------------------------------'''

#import built-in modules
import os,sys
if sys.version_info[0] != 2 or sys.version_info[1] != 7:
	print >>sys.stderr, "\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " RSeQC needs python2.7!\n"
	sys.exit()

import re
import string
from optparse import OptionParser
import warnings
import string
import collections
import math
import sets
import numpy as np
from time import strftime
import operator
import subprocess

#import third-party modules
from bx.bitset import *
from bx.bitset_builders import *
from bx.intervals import *

#import my own modules
from qcmodule import SAM
#changes to the paths

#changing history to this module


__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="2.6.5"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


def printlog (mesg):
	'''print progress into stderr and log file'''
	mesg="@ " + strftime("%Y-%m-%d %H:%M:%S") + ": " + mesg
	LOG=open('class.log','a')
	print >>sys.stderr,mesg
	print >>LOG,mesg

def normalize(lst):
	'''normalize all numbers between 0 and 1'''
	norm_lst=[]
	if max(lst) == min(lst):
		return norm_lst
	if max(lst) - min(lst)==0:
		return norm_lst
	for i in lst:
		norm_lst.append(  (i-min(lst))/(max(lst)-min(lst))  )  
	return norm_lst

def square_error(lst):
	'''transform list into normalized squared error (squared error divided by range)'''
	SE = []
	true_rpkm = lst[-1]
	rang = max(lst) - min(lst)
	if true_rpkm == 0:
		return None
	if rang == 0:
		return None
	for i in lst:
		SE.append(  abs(i - true_rpkm)/true_rpkm  )  
	return SE


def show_saturation (infile,outfile,rpkm_cut=0.01):
	
	RPKM_values = collections.defaultdict(list)
	RPKM_mean = {}
	gene_count = 0
	Quan = {'Q1':[0,0.25],'Q2':[0.25,0.5],'Q3':[0.5,0.75],'Q4':[0.75,1]}
	ROUT = open(outfile,'w')
	
	for line in open(infile):
		line=line.strip()
		fields=line.split()
		if fields[0].startswith('#'):
			head = [i.replace('%','') for i in fields[6:]]
			continue
		mykey = '\t'.join(fields[0:6])
		myvalue = [float(i) for i in fields[6:]]
		if max(myvalue) == 0: continue
		if max(myvalue) - min(myvalue) == 0: continue
		if np.mean(myvalue) < rpkm_cut: continue
		
		RPKM_values[mykey] = square_error(myvalue)
		RPKM_mean[mykey] = np.mean(myvalue)
		gene_count += 1
		if (len(head)==0):
			print >>sys.stderr, "No head line found, exit."
			sys.exit(1)
	
	
	print >>ROUT, "pdf('%s')" % (outfile.replace('.r','.pdf'))
	print >>ROUT, 'par(mfrow=c(2,2))'
	for quantile in sorted(Quan):
		line_count = 0
		norm_RPKM = collections.defaultdict(list)
		for k,v in sorted (RPKM_mean.iteritems(), key=operator.itemgetter(1)):	
			line_count += 1
			if (line_count > gene_count * Quan[quantile][0]) and (line_count <= gene_count * Quan[quantile][1]):
				for i,j in enumerate(RPKM_values[k]):
					norm_RPKM[head[i]].append(str(j))
		print >>ROUT,"name=c(%s)" % (','.join(head[:-1]))
		for i in head[:-1]:
			print >>ROUT, "S%s=c(%s)" % (i, ','.join(norm_RPKM[i]))
		print >>ROUT, "boxplot(%s,names=name,outline=F,ylab='Percent Relative Error',main='%s',xlab='Resampling percentage')" % (','.join(['100*S' + i for i in head[:-1]]),quantile)
	print >>ROUT, 'dev.off()'
				

def main():
	usage="%prog [options]" + '\n' + __doc__ + "\n"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input-file",action="store",type="string",dest="input_file",help="Alignment file in BAM or SAM format. [required]")
	parser.add_option("-o","--out-prefix",action="store",type="string",dest="output_prefix",help="Prefix of output files(s). [required]")
	parser.add_option("-r","--refgene",action="store",type="string",dest="refgene_bed",help="Reference gene model in bed fomat. [required]")
	parser.add_option("-d","--strand",action="store",type="string",dest="strand_rule",default=None,help="How read(s) were stranded during sequencing. For example: --strand='1++,1--,2+-,2-+' means that this is a pair-end, strand-specific RNA-seq, and the strand rule is: read1 mapped to '+' => parental gene on '+'; read1 mapped to '-' => parental gene on '-'; read2 mapped to '+' => parental gene on '-'; read2 mapped to '-' => parental gene on '+'.  If you are not sure about the strand rule, run \'infer_experiment.py' default=%default (Not a strand specific RNA-seq data)")
	parser.add_option("-l","--percentile-floor",action="store",type="int",dest="percentile_low_bound",default=5, help="Sampling starts from this percentile. A integer between 0 and 100. default=%default")
	parser.add_option("-u","--percentile-ceiling",action="store",type="int",dest="percentile_up_bound",default=100, help="Sampling ends at this percentile. A integer between 0 and 100. default=%default")
	parser.add_option("-s","--percentile-step",action="store",type="int",dest="percentile_step",default=5, help="Sampling frequency. Smaller value means more sampling times. A integer between 0 and 100. default=%default")	
	parser.add_option("-c","--rpkm-cutoff",action="store",type="float",dest="rpkm_cutoff",default=0.01, help="Transcripts with RPKM smaller than this number will be ignored in visualization plot. default=%default")
	parser.add_option("-q","--mapq",action="store",type="int",dest="map_qual",default=30,help="Minimum mapping quality (phred scaled) for an alignment to be called \"uniquely mapped\". default=%default")
	
	(options,args)=parser.parse_args()

	if not (options.output_prefix and options.input_file):
		parser.print_help()
		sys.exit(0)
	if options.percentile_low_bound <0 or options.percentile_low_bound >100:
		print >>sys.stderr, "percentile_low_bound must be larger than 0 and samller than 100"
		sys.exit(0)
	if options.percentile_up_bound <0 or options.percentile_up_bound >100:
		print >>sys.stderr, "percentile_up_bound must be larger than 0 and samller than 100"
		sys.exit(0)
	if options.percentile_up_bound < options.percentile_low_bound:
		print >>sys.stderr, "percentile_up_bound must be larger than percentile_low_bound"
		sys.exit(0)
	if options.percentile_step <0 or options.percentile_step > options.percentile_up_bound:
		print >>sys.stderr, "percentile_step must be larger than 0 and samller than percentile_up_bound"
		sys.exit(0)
	if os.path.exists(options.input_file):
		obj = SAM.ParseBAM(options.input_file)
		obj.saturation_RPKM(outfile=options.output_prefix, refbed=options.refgene_bed, sample_start=options.percentile_low_bound,sample_end=options.percentile_up_bound,sample_step=options.percentile_step,strand_rule=options.strand_rule, q_cut  = options.map_qual)
		show_saturation(infile=options.output_prefix + ".eRPKM.xls", outfile=options.output_prefix + ".saturation.r",rpkm_cut = options.rpkm_cutoff)
		try:
			subprocess.call("Rscript " + options.output_prefix + ".saturation.r", shell=True)
		except:
			pass
	else:
		print >>sys.stderr, '\n\n' + options.input_file + " does NOT exists" + '\n'
		#parser.print_help()
		sys.exit(0)
		


if __name__ == '__main__':
	main()



#!/usr/bin/env python
'''Manipulate two bigwig files'''
import os,sys
if sys.version_info[0] != 2 or sys.version_info[1] != 7:
	print >>sys.stderr, "\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " RSeQC needs python2.7!\n"
	sys.exit()

import string
from optparse import OptionParser

from bx.bbi.bigwig_file import BigWigFile
from qcmodule import BED
from qcmodule import twoList
import numpy
__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="2.6.5"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"

def load_chromsize(file):
	'''read chrom.size file'''
	chromSize={}
	for line in open(file,'r'):
		if line.startswith('#'):continue
		if not line.strip():continue
		fields = line.strip().split()
		chromSize[fields[0]] = int(fields[1])
	return chromSize

def main():
	usage="%prog [options]"
	parser = OptionParser(usage,version="%prog " + __version__)
	
	parser.add_option("-i","--bwfile1",action="store",type="string",dest="BigWig_File1",help="One BigWig file")
	parser.add_option("-j","--bwfile2",action="store",type="string",dest="BigWig_File2",help="Another BigWig file")
	parser.add_option("-a","--action",action="store",type="string",dest="action",help='After pairwise align two bigwig files, perform the follow actions (Only select one keyword):"Add" = add signals. "Average" = average signals. "Division"= divide bigwig2 from bigwig1. Add 1 to both bigwig. "Max" = pick the signal that is larger. "Min" = pick the signal that is smaller. "Product" = multiply signals. "Subtract" = subtract signals in 2nd bigwig file from the corresponiding ones in the 1st bigwig file. "geometricMean" = take the geometric mean of signals.')
	parser.add_option("-o","--output",action="store",type="string",dest="output_wig",help="Output wig file")
	parser.add_option("-s","--chromSize",action="store",type="string",dest="chromSize",help="Chromosome size file. Tab or space separated text file with 2 columns: first column is chromosome name, second column is size of the chromosome.")
	parser.add_option("-c","--chunk",action="store",type="int",dest="chunk_size",default=100000,help="Chromosome chunk size. Each chomosome will be cut into samll chunks of this size. Decrease chunk size will save more RAM. default=%default (bp)")
	parser.add_option("-m","--min_signal",action="store",type="float",dest="min_score",default=0.0,help="To redude the size of output wigfile, genomic positions with signal value smaller than (<) this threshold will be filtered out. default=%default")
	(options,args)=parser.parse_args()
	
	if not (options.BigWig_File1 and options.BigWig_File2  and options.output_wig and options.chromSize):
		parser.print_help()
		sys.exit(0)
	OUT=open(options.output_wig,'w')
	bw1 = BigWigFile( file=open(options.BigWig_File1) )
	bw2 = BigWigFile( file=open(options.BigWig_File2) )
	chrom_sizes = load_chromsize(options.chromSize)
	for chr_name, chr_size in chrom_sizes.items():		#iterate each chrom
		print >>sys.stderr, "Processing " + chr_name + " ..."
		OUT.write('variableStep chrom='+chr_name+'\n')
		for interval in BED.tillingBed(chrName = chr_name,chrSize = chr_size,stepSize = options.chunk_size):
			coord = interval[1]
			try:
				bw_signal1 = bw1.get_as_array(chr_name,interval[1],interval[2])
			except:
				bw_signal1 = numpy.array()
			try:
				bw_signal2 = bw2.get_as_array(chr_name,interval[1],interval[2])
			except:
				bw_signal2 = numpy.array()
			if bw_signal1 is None and bw_signal2 is None:
				continue
			if numpy.isnan(numpy.nansum(bw_signal1)) and numpy.isnan(numpy.nansum(bw_signal2)):
				continue
			if len(bw_signal1) == 0 and len(bw_signal2) == 0:
				continue
			bw_signal1 = numpy.nan_to_num( bw_signal1 )
			bw_signal2 = numpy.nan_to_num( bw_signal2 )
		
			call_back = getattr(twoList,options.action)
			for v in call_back(bw_signal1,bw_signal2):
				coord +=1
				if v >= options.min_score: print >>OUT, "%d\t%.2f" % (coord,v)

if __name__=='__main__':
	main()



import math
def point_poip(actual, mean):
	'''give poisson pvalue. P[obs ==mean]'''
	# naive:   math.exp(-mean) * mean**actual / factorial(actual)
	# iterative, to keep the components from getting too large or small:
	p = math.exp(-mean)
	for i in xrange(actual):
		p *= mean
		p /= i+1
	return p

def cumu_poip(num, mean,logp=False):
	'''give poisson pvalue P[obs >=mean]'''
	s=0.0
	for i in range(0,num+1):
		s += point_poip(i,mean)
	if logp is True:
		try:
			return -10*math.log10(1-s)
		except:
			return 3000
	else:
		return 1-s
	



import os,sys
from os.path import abspath,join, getsize
"""
Get bam files from input, input could be:
1) directory that containing one or more bam files
2) plain text file containing one or more bam file paths
3) a single bam file
4) ',' separated multiple bam files

in all cases, the index .bai file(s) should be exist in the same location. eg, if test.bam
exists, test.bam.bai must also exist.
"""

def isbamfile(infile):
	'''check if it is bam file, if it is empty and if the .bai file exists'''
	if os.path.isfile(infile) and infile[-4:].lower() == '.bam':
		if getsize(infile) != 0:
			if os.path.isfile(infile + '.bai'):
				return True
			else:
				print >>sys.stderr, "Warning: %s.bai does not exist! Skip it." % (infile)
				return False
		else:
			print >>sys.stderr, "The size of %s is 0! Skip it." % (infile)
			return False
	else:
		return False	

def get_bam_files (input,printit=False):
	bam_files = []	
	
	#dir
	if os.path.isdir(input):
		for root, directories, files in os.walk(input,followlinks=True):
			full_names = [join(abspath(root), name) for name in files]
			for fn in full_names:
				if isbamfile(fn):
					bam_files.append(fn)
	#single bam file
	elif isbamfile(input):
		bam_files.append(input)
	#plain text file
	elif os.path.isfile(input):
		try:
			for line in open(input):
				line = line.strip()
				if line.startswith('#'):continue
				if isbamfile(line):
					bam_files.append(line)
		except:
			pass
	else:
		tmp = input.split(',')
		if len(tmp) <2: pass
		for i in tmp:
			if isbamfile(i):
				bam_files.append(i)
	
	if printit:
		for i in bam_files:
			print i		
	return bam_files


if __name__ == '__main__':
	get_bam_files(sys.argv[1])


#!/usr/bin/env python

#import built-in modules
import os,sys
import re
import string
from optparse import OptionParser
import warnings
import string
import collections
import math

#import third-party modules

#changes to the paths

#changing history to this module
#05/26/2011: suppport multiple spliced mapped reads

__author__ = "Liguo Wang"
__copyright__ = "Copyright 2010, Wei Li's Lab"
__credits__ = []
__license__ = "GPL"
__version__ = "1.1.0"
__maintainer__ = "Liguo Wang"
__email__ = "liguow@bcm.edu"
__status__ = "Development" #Prototype or Production


def RSS(arg):
	'''calculate Square root of sum of square. Input is ',' separated numbers'''
	lst=arg.split(',')
	lst_sum=0
	for i in [ int(i)**2 for i in lst]:
		lst_sum += i 
	#nsr=10*math.log10((1+noi_sum**0.5)/(1+sig_sum**0.5))
	return lst_sum**0.5
	
def H_mean(arg):
	'''calculate harmornic mean. Input is ',' separated numbers'''
	lst=[1/float(i) for i in arg.split(',') if float(i) !=0]
	if len(lst) == 0:
		return "NA"
	else:
		return len(lst)/(sum(lst))

def shannon_entropy(arg):
	'''calculate shannon's entropy (or Shannon-Wiener index).'''
	lst=arg
	lst=[float(i) for i in lst if float(i)>0]
	entropy=0.0
	for i in lst:
		entropy += (i/sum(lst)) * math.log((i/sum(lst)))
	if entropy == 0:
		return 0
	else:
		return -entropy

	
def shannon_entropy_es(arg):
	'''calculate estimator of shannon's entropy (Chao & Shen, 2003)'''
	lst=arg
	lst=[float(i) for i in lst if float(i)>0]
	if sum(lst)<=0 or min(lst)<0:return "NA"	#if there is no fragmental splicing
	if (len(lst)==1): return 0					#if there is only 1 fragmental splicing
	lst.append(2)
	
	#estimate C_bar
	singleton=0
	entropy=0.0
	for i in lst:
		if i ==1:singleton +=1
	
	C_bar = 1- (singleton/sum(lst))
	for i in lst:entropy += ( (C_bar*i/sum(lst)) * math.log((C_bar*i/sum(lst))) )/(1-(1-C_bar*i/sum(lst))**sum(lst))
	if entropy == 0:
		return 0
	else:
		return -entropy

def shannon_entropy_ht(arg):
	'''calculate estimator of shannon's entropy based on Horzitz-Thompson'''
	lst=arg.split(',')
	lst=[float(i) for i in lst if float(i)>0]
	if sum(lst)<=0 or min(lst)<0:return "NA"	#if there is no fragmental splicing
	if (len(lst)==1): return 0					#if there is only 1 fragmental splicing
	
	#estimate C_bar
	entropy=0.0
	for i in lst:
		entropy += ( (i/sum(lst)) * math.log((i/sum(lst))) )/(1-(1-i/sum(lst))**sum(lst))
	return -entropy
	
def simpson_index(arg):
	'''calculate Gini-Simpson's index. Input is ',' separated numbers'''
	lst=arg.split(',')
	lst=[float(i) for i in lst if float(i)>0]
	simpson=0.0
	
	try:
		for i in lst:
			simpson = simpson + (i/sum(lst))**2
		return 1-simpson
	except: return 0
	
def simpson_index_es(arg):
	'''calculate estimator Gini-Simpson's index. Input is ',' separated numbers'''
	lst=arg.split(',')
	lst=[float(i) for i in lst if float(i)>0]
	simpson=0.0
	
	try:
		for i in lst:
			simpson = simpson + i*(i-1)
		return 1- (simpson/(sum(lst)*(sum(lst)-1)))
	except: return 0
	
def Hill_number(arg,qvalue=1):
	'''Calculate real diversity (Hill's number). Input is ',' separated numbers. qvalue is the only
	parameter for Hill's function. When q=1, it return exp(H) which is the effective number of junctions
	calculated by Shannon's entropy. When q<1, Hill's function was favors low frequency junctions. 
	When q>1, Hill's function was favors high frequency junctions (common junctions). Simpon's Index
	is particular case of Hill's function as q=2'''
	
	lst=arg.split(',')
	lst=[float(i) for i in lst if float(i)>0]
	freq=[(i/sum(lst))**qvalue for i in lst]
	try:
		return (sum(freq))**(1/(1-qvalue))
	except:
		return math.exp(shannon_entropy(arg))
import math
import functools

def percentile(N, percent, key=lambda x:x):
    """
    Find the percentile of a list of values.

    @parameter N - is a list of values. Note N MUST BE already sorted.
    @parameter percent - a float value from 0 to 100.
    @parameter key - optional key function to compute value from each element of N.

    @return - the percentile of the values
    """
    if not N:
        return None
    k = (len(N)-1) * percent/100.0
    f = math.floor(k)
    c = math.ceil(k)
    if f == c:
        return key(N[int(k)])
    d0 = key(N[int(f)]) * (c-k)
    d1 = key(N[int(c)]) * (k-f)
    return d0+d1
	
def percentile_list(N):
	"""
	Find the percentile of a list of values.
	@parameter N - is a list of values. Note N MUST BE already sorted.
    @return - the list of percentile of the values
	"""
	if not N:return None
	if len(N) <100: return N
	per_list=[]
	for i in range(1,101):
		k = (len(N)-1) * i/100.0
		f = math.floor(k)
		c = math.ceil(k)
		if f == c:
			per_list.append( int(N[int(k)])  )
		else:
			d0 = N[int(f)] * (c-k)
			d1 = N[int(c)] * (k-f)
			per_list.append(int(round(d0+d1)))	
	return per_list



#!/usr/bin/env python
'''manipulate fasta for fastq format files.'''

#import built-in modules
import re
import sys
from string import maketrans
from random import shuffle
from heapq import nlargest
#import third-party modules

#changes to the paths

#changing history to this module


__author__ = "Liguo Wang"
__copyright__ = "Copyright 2010, Wei Li's Lab"
__credits__ = []
__license__ = "GPL"
__version__ = "1.0.1"
__maintainer__ = "Liguo Wang"
__email__ = "liguow@bcm.edu"
__status__ = "Development" #Prototype or Production


def S_diff(lst):
	'''Given a list of int or float, calculate S_diff and S_point'''
	
	S_avg = sum(lst) / len(lst)
	S_dist = [i-S_avg for i in lst]	#distance to average
	S_cum=[]	#list of cumulative sum
	S_cum.append(0)
	for i in range(0,len(S_dist)):
		S_cum.append(S_cum[i] + S_dist[i])
	return [nlargest(1,range(0,len(S_cum)),key=lambda i: S_cum[i]),(max(S_cum) - min(S_cum))]
	#return the index of maximum_diff index, and maximum_diff

def bootstrap(lst,obs,rep=1000):
	'''Given a list of int or float (lst) and an observation value(obs). calcualte the chance (pvalue) 
	of getting this observation through bootstrapping.'''
	
	shuffled_diff=[]
	count=0
	tmp=lst
	for i in range(0,rep):
		shuffle(tmp)
		shuffled_diff.append(S_diff(tmp))
	
	for i in sorted(shuffled_diff):
		if (i>=obs):
			count += 1 
	if count/rep <0.5:
		return count/rep
	else:
		return 1- count/rep
